@@ -19,6 +19,7 @@ import (
 	"math/rand"
 	"net/http"
 	"net/http/httptest"
+	"net/netip"
 	"net/url"
 	"sort"
 	"strings"
@@ -27,7 +28,7 @@ import (
 
 	"github.com/klauspost/compress/zstd"
 	"go4.org/mem"
-	"inet.af/netaddr"
+	"tailscale.com/net/netaddr"
 	"tailscale.com/net/tsaddr"
 	"tailscale.com/smallzstd"
 	"tailscale.com/tailcfg"
@@ -311,7 +312,7 @@ func (s *Server) AddFakeNode() {
 	r := nk.Raw32()
 	id := int64(binary.LittleEndian.Uint64(r[:]))
 	ip := netaddr.IPv4(r[0], r[1], r[2], r[3])
-	addr := netaddr.IPPrefixFrom(ip, 32)
+	addr := netip.PrefixFrom(ip, 32)
 	s.nodes[nk] = &tailcfg.Node{
 		ID:                tailcfg.NodeID(id),
 		StableID:          tailcfg.StableNodeID(fmt.Sprintf("TESTCTRL%08x", id)),
@@ -320,8 +321,8 @@ func (s *Server) AddFakeNode() {
 		Key:               nk,
 		MachineAuthorized: true,
 		DiscoKey:          dk,
-		Addresses:         []netaddr.IPPrefix{addr},
-		AllowedIPs:        []netaddr.IPPrefix{addr},
+		Addresses:         []netip.Prefix{addr},
+		AllowedIPs:        []netip.Prefix{addr},
 	}
 	// TODO: send updates to other (non-fake?) nodes
 }
@@ -474,10 +475,10 @@ func (s *Server) serveRegister(w http.ResponseWriter, r *http.Request, mkey key.
 
 	machineAuthorized := true // TODO: add Server.RequireMachineAuth
 
-	v4Prefix := netaddr.IPPrefixFrom(netaddr.IPv4(100, 64, uint8(tailcfg.NodeID(user.ID)>>8), uint8(tailcfg.NodeID(user.ID))), 32)
-	v6Prefix := netaddr.IPPrefixFrom(tsaddr.Tailscale4To6(v4Prefix.IP()), 128)
+	v4Prefix := netip.PrefixFrom(netaddr.IPv4(100, 64, uint8(tailcfg.NodeID(user.ID)>>8), uint8(tailcfg.NodeID(user.ID))), 32)
+	v6Prefix := netip.PrefixFrom(tsaddr.Tailscale4To6(v4Prefix.Addr()), 128)
 
-	allowedIPs := []netaddr.IPPrefix{
+	allowedIPs := []netip.Prefix{
 		v4Prefix,
 		v6Prefix,
 	}
@@ -760,10 +761,10 @@ func (s *Server) MapResponse(req *tailcfg.MapRequest) (res *tailcfg.MapResponse,
 		return res.Peers[i].ID < res.Peers[j].ID
 	})
 
-	v4Prefix := netaddr.IPPrefixFrom(netaddr.IPv4(100, 64, uint8(tailcfg.NodeID(user.ID)>>8), uint8(tailcfg.NodeID(user.ID))), 32)
-	v6Prefix := netaddr.IPPrefixFrom(tsaddr.Tailscale4To6(v4Prefix.IP()), 128)
+	v4Prefix := netip.PrefixFrom(netaddr.IPv4(100, 64, uint8(tailcfg.NodeID(user.ID)>>8), uint8(tailcfg.NodeID(user.ID))), 32)
+	v6Prefix := netip.PrefixFrom(tsaddr.Tailscale4To6(v4Prefix.Addr()), 128)
 
-	res.Node.Addresses = []netaddr.IPPrefix{
+	res.Node.Addresses = []netip.Prefix{
 		v4Prefix,
 		v6Prefix,
 	}
@@ -859,13 +860,13 @@ func filterInvalidIPv6Endpoints(eps []string) []string {
 }
 
 func keepClientEndpoint(ep string) bool {
-	ipp, err := netaddr.ParseIPPort(ep)
+	ipp, err := netip.ParseAddrPort(ep)
 	if err != nil {
 		// Shouldn't have made it this far if we unmarshalled
 		// the incoming JSON response.
 		return false
 	}
-	ip := ipp.IP()
+	ip := ipp.Addr()
 	if ip.Zone() != "" {
 		return false
 	}
