@@ -5,9 +5,11 @@
 // Package dnstype defines types for working with DNS.
 package dnstype
 
-//go:generate go run tailscale.com/cmd/cloner --type=Resolver --clonefunc=true
+//go:generate go run tailscale.com/cmd/viewer --type=Resolver --clonefunc=true
 
-import "inet.af/netaddr"
+import (
+	"net/netip"
+)
 
 // Resolver is the configuration for one DNS resolver.
 type Resolver struct {
@@ -25,22 +27,22 @@ type Resolver struct {
 	// BootstrapResolution may be empty, in which case clients should
 	// look up the DoT/DoH server using their local "classic" DNS
 	// resolver.
-	BootstrapResolution []netaddr.IP `json:",omitempty"`
+	BootstrapResolution []netip.Addr `json:",omitempty"`
 }
 
 // IPPort returns r.Addr as an IP address and port if either
 // r.Addr is an IP address (the common case) or if r.Addr
 // is an IP:port (as done in tests).
-func (r *Resolver) IPPort() (ipp netaddr.IPPort, ok bool) {
+func (r *Resolver) IPPort() (ipp netip.AddrPort, ok bool) {
 	if r.Addr == "" || r.Addr[0] == 'h' || r.Addr[0] == 't' {
 		// Fast path to avoid ParseIP error allocation for obviously not IP
 		// cases.
 		return
 	}
-	if ip, err := netaddr.ParseIP(r.Addr); err == nil {
-		return netaddr.IPPortFrom(ip, 53), true
+	if ip, err := netip.ParseAddr(r.Addr); err == nil {
+		return netip.AddrPortFrom(ip, 53), true
 	}
-	if ipp, err := netaddr.ParseIPPort(r.Addr); err == nil {
+	if ipp, err := netip.ParseAddrPort(r.Addr); err == nil {
 		return ipp, true
 	}
 	return
