@@ -9,5 +9,7 @@ import "os"
 
 // On reports whether the current binary is executing on a CI system.
 func On() bool {
-	return os.Getenv("GITHUB_ACTIONS") != ""
+	// CI env variable is set by GitHub.
+	// https://docs.github.com/en/actions/learn-github-actions/environment-variables#default-environment-variables
+	return os.Getenv("GITHUB_ACTIONS") != "" || os.Getenv("CI") == "true"
 }
